<?php

/*
 * @package RS-MINI
 * @copyright (c) 2015 Alexey Glumov aka Rio-Shaman (support@rio-shaman.ru)
 * @license GNU General Public License version 2; see LICENSE.txt
 *
 */
 

namespace core;

if(!defined('RS-MINI')) die();


/*
 * dom v 1.0
 * 
 * Класс хранит данные для передачи их представлению
 *
 */

class dom
{   
    
    /*
     * массив с атрибутами узла
     *
     * @var    - array
     * @access - public
     */
    
    public $attr = array();
    
    /*
     * массив с узлами
     *
     * @var    - array
     * @access - public
     */
    
    public $node = array();
    
    /*
     * имя узла
     *
     * @var    - string
     * @access - public
     */
    
    public $name = NULL;
    
    /*
     * добавляем новый узел. если в качестве параметра был передан объект dom то
     * в массив $this->node запишится этот объект. если в качестве параемтера передено строка (т.е. имя нового узла)
     * то на основе этого имени поднимется новая копия объекта класса dom
     * 
     * @access - public
     * @return - возвращает созданный узел
     * 
     * @param string/object $data - данные содержащие имя нового узла или объект дома
     */

    public function addNode($data)
    {
        $node           = (!is_object($data)) ? new dom()             : $data;
        $this->node[]   = (!is_object($data)) ? $node->setName($data) : $node;
        
        return $node;
    }

    /*
     * прописываем новый атрибут в текущий объект класса dom
     * 
     * @access - public
     * @return - сам себя
     * 
     * @param string name  - имя атрибута
     * @param string value - значение атрибута
     */

    public function addAttr($name, $value)
    {
        $this->attr[$name] = $value;

        return $this;
    }
    
    /*
     * получаем узел (объект) по имени из массива $this->node.
     * 
     * @access - public
     * @return - узел или FALSE
     * 
     * @param string name - имя узла
     */
     
    public function getNode($name)
    {
        // листаем дочернии узлы
        foreach ($this->node as $node)
            // если имя из объекта совпадает с искомым именем
            if ($node->name == $name)
                return $node;
                
        // по умолчанию метод возвращает фалс
        return FALSE;
    }

    /*
     * получаем значение атрибута. Узел выберается с помощью метода getNode() или getChildren()
     * 
     * @access - public
     * @return - значение атрибута или FALSE в случае неудачи
     * 
     * @param string name - имя атрибута
     */
    
    public function getAttr($name)
    {
        return (isset($this->attr[$name])) ? $this->attr[$name] : FALSE;
    }

    /*
     *  получаем весь дом целиком в виде массива
     * 
     * @access - public
     * @return - массив с всеми узлами и их атрибутами
     * 
     */

    public function getDom()
    {
        return $this->collectNode($this->node);
    }
    
    /*
     * рекурсивно собираем узлы (вспомогательный метод метода getDom()) вертая массив вот такой структуры
     * [0]
     *     [name] => имя,
     *     [attr] => массив атрибутов,
     *     [node] =>
     *            [0]
     *                [name] => имя,
     *                [attr] => массив атрибутов,
     *                [node] => array()
     *            [1]
     *                [name] => имя,
     *                [attr] => массив атрибутов,
     *                [node] => 
     *                       [0]
     *                           [name] => имя,
     *                           [attr] => массив атрибутов,
     *                           [node] => array()
     * 
     * @access  - private
     * @return  - массив с узлом
     * 
     * @param array nodeList - список узлов (объектов dom)
     */
    
    private function collectNode($nodeList)
    {
        $nodes = array();
        
        foreach ($nodeList as $key => $node)            
            $nodes[] = array(
                'name'  => $node->name,
                'attr'  => $node->attr,
                'node'  => (count($node->node) > 0) ? $this->collectNode($node->node) : array()
            );
        
        return $nodes;      
    }

    /*
     * получаем массив с вложенными узлами (объекты) из текущего узла
     * 
     * @access - public
     * @return - массив с узлами или FALSE
     * 
     * @param string name - имя узла
     */
    
    public function getChildren($name)
    {
        // массив с выбранными узлами
        $nodeList = array();
        
        // листаем дочернии узлы
        foreach ($this->node as $node)
            // если имя из объекта совпадает с искомым именем
            if ($node->name == $name)
                // заполняем выборку узлом
                $nodeList[] = $node;
        
        // если небыло выбрано ни одного узла, то фалсе. в противном случае список узлов
        return (empty($nodeList)) ? FALSE : $nodeList;
    }

    /*
     * проверяем есть ли узлы в выбранном узле
     * 
     * @access - public
     * @return - число узлов если они есть, или FALSE если нет
     * 
     * @param string name - имя узла
     */
     
    public function count($name)
    {
        // массив с выбранными узлами
        $nodeList = array();
        
        // листаем дочернии узлы
        foreach ($this->node as $node)
            // если имя из объекта совпадает с искомым именем
            if ($node->name == $name)
                // заполняем выборку узлом
                $nodeList[] = $node;
        
        // если небыло выбрано ни одного узла, то фалсе. в противном случае кол-во узлов
        return (empty($nodeList)) ? FALSE : count($nodeList);
    }
    
    /*
     * метод устанавливает имя узла
     * 
     * @access - private
     * @return - сам себя
     * 
     * @param string name - имя узла
     */
    
    private function setName($name)
    {
        $this->name = $name;
        
        return $this;
    }
}
